use rand::seq::SliceRandom;

use crate::guess::*;
use crate::constraints;

pub trait Guesser<const N: usize> {
    fn push(&mut self, _result: GuessResult<N>) {}
    fn guess(&self) -> Word<N>;
}

pub struct RandomConstraintGuesser<'a, const N: usize> {
    possible_words: Dictionary<N>,
    history: GuessHistory<N>,
    constraints: &'a mut dyn constraints::Constraints<N>,
}

impl<'a, const N: usize> RandomConstraintGuesser<'a, N> {
    pub fn new(
        dictionary: &Dictionary<N>,
        constraints: &'a mut dyn constraints::Constraints<N>,
    ) -> Self {
        Self {
            possible_words: dictionary.to_vec(),
            history: GuessHistory::new(),
            constraints,
        }
    }
}

impl<'a, const N: usize> Guesser<N> for RandomConstraintGuesser<'a, N> {
    fn guess(&self) -> Word<N> {
        *self.possible_words.choose(&mut rand::thread_rng()).unwrap()
    }

    fn push(&mut self, result: GuessResult<N>) {
        self.history.push(result);
        self.constraints.push(result);
        let constraints = &self.constraints;
        self.possible_words.retain(|&word| constraints.check(word));
    }
}
