use std::convert::TryInto;

#[macro_export]
macro_rules! word {
    ( $x:expr ) => {
        {
            $x.chars()
                .collect::<Vec<char>>()
                .as_slice()
                .try_into()
                .unwrap()
        }
    }
}

mod guess;
use guess::*;

mod constraints;
mod game;
mod guesser;
mod evaluator;


fn play<const N: usize>(dictionary: &Dictionary<N>) {
    let mut constraints = constraints::Total::new();
    let mut game = game::Game {
        evaluator: &mut evaluator::AdversarialEvaluator::new(dictionary),
        guesser: &mut guesser::RandomConstraintGuesser::new(dictionary, &mut constraints),
    };
    loop {
        let result = game.next();
        println!("{}", result);
        if result.correct() {
            return
        }
    }
}

fn load_words<const N: usize>(contents: &'static str) -> Vec<Word<N>> {
    contents
        .split("\n")
        .map(|line| word!(line))
        .collect::<Vec<_>>()
}

fn main() {
    let common_words: Vec<Word<5>> = load_words(include_str!("words/common.txt"));
    loop {
        play(&common_words);
    }
}
